package App::POS::Config;

use strict;
use warnings;
use Config::General;
use FindBin qw( $Bin );
use File::Spec;
use Data::Dumper;
use POSIX qw( strftime );

use base qw( App::Class );

sub file {
  my $self = shift;
  @_ ? $self->{file} = shift : $self->{file};
}

sub _init_array_fields {
  my $self = shift;
  my $config = shift;

  if (
    exists $config->{AREA} &&
    ! ref $config->{AREA}
  ) {
    my $tmp = delete $config->{AREA};
    push @{$config->{AREA}}, $tmp;
  }

  if (
    exists $config->{QUIZ_QUESTIONS} &&
    ! ref $config->{QUIZ_QUESTIONS}
  ) {
    my $tmp = delete $config->{QUIZ_QUESTIONS};
    push @{$config->{QUIZ_QUESTIONS}}, $tmp;
  }

  if (
    exists $config->{PRINTING_ZONE_LINK} &&
    ! ref $config->{PRINTING_ZONE_LINK}
  ) {
    my $tmp = delete $config->{PRINTING_ZONE_LINK};
    push @{$config->{PRINTING_ZONE_LINK}}, $tmp;
  }

  if (
    exists $config->{QUEUES} &&
    ! ref $config->{QUEUES}
  ) {
    my $tmp = delete $config->{QUEUES};
    push @{$config->{QUEUES}}, $tmp;
  }

  if (
    exists $config->{ORDER_SEPARATORS} &&
    ! ref $config->{ORDER_SEPARATORS}
  ) {
    my $tmp = delete $config->{ORDER_SEPARATORS};
    push @{$config->{ORDER_SEPARATORS}}, $tmp;
  }

  if (
    exists $config->{PRINT_RECEIPT_PRINTER_VIAS} &&
    ! ref $config->{PRINT_RECEIPT_PRINTER_VIAS}
  ) {
    my $tmp = delete $config->{PRINT_RECEIPT_PRINTER_VIAS};
    push @{$config->{PRINT_RECEIPT_PRINTER_VIAS}}, $tmp;
  }

  if (
    exists $config->{AREA_HAPPY_HOUR} &&
    ! ref $config->{AREA_HAPPY_HOUR}
  ) {
    my $tmp = delete $config->{AREA_HAPPY_HOUR};
    push @{$config->{AREA_HAPPY_HOUR}}, $tmp;
  }

  if (
    exists $config->{GROUP_REQUESTS_BY_PRINTER_SPECIFIC} &&
    ! ref $config->{GROUP_REQUESTS_BY_PRINTER_SPECIFIC}
  ) {
    my $tmp = delete $config->{GROUP_REQUESTS_BY_PRINTER_SPECIFIC};
    push @{$config->{GROUP_REQUESTS_BY_PRINTER_SPECIFIC}}, $tmp;
  }

  if (
    exists $config->{ORDER_PRINTER_ERRORS_CONFIG} &&
    ! ref $config->{ORDER_PRINTER_ERRORS_CONFIG}
  ) {
    my $tmp = delete $config->{ORDER_PRINTER_ERRORS_CONFIG};
    push @{$config->{ORDER_PRINTER_ERRORS_CONFIG}}, $tmp;
  }

  if (
    exists $config->{SAFT_SERIES_CODES} &&
    ! ref $config->{SAFT_SERIES_CODES}
  ) {
    my $tmp = delete $config->{SAFT_SERIES_CODES};
    push @{$config->{SAFT_SERIES_CODES}}, $tmp;
  }

  if (
    exists $config->{AREA_CHECK_VALIDATION} &&
    ! ref $config->{AREA_CHECK_VALIDATION}
  ) {
    my $tmp = delete $config->{AREA_CHECK_VALIDATION};
    push @{$config->{AREA_CHECK_VALIDATION}}, $tmp;
  }

  if (
    exists $config->{AREA_HAPPY_HOUR} &&
    ! ref $config->{AREA_HAPPY_HOUR}
  ) {
    my $tmp = delete $config->{AREA_HAPPY_HOUR};
    push @{$config->{AREA_HAPPY_HOUR}}, $tmp;
  }

  if (
    exists $config->{QUEUES} &&
    ! ref $config->{QUEUES}
  ) {
    my $tmp = delete $config->{QUEUES};
    push @{$config->{QUEUES}}, $tmp;
  }

  if (
    exists $config->{STATION_CASHIERS} &&
    ! ref $config->{STATION_CASHIERS}
  ) {
    my $tmp = delete $config->{STATION_CASHIERS};
    push @{$config->{STATION_CASHIERS}}, $tmp;
  }

  if (
    exists $config->{CASHIERS_BY_STATION} &&
    ! ref $config->{CASHIERS_BY_STATION}
  ) {
    my $tmp = delete $config->{CASHIERS_BY_STATION};
    push @{$config->{CASHIERS_BY_STATION}}, $tmp;
  }

  if (
    exists $config->{REDIRECT_TO_NETWORK_PRINTERS} &&
    ! ref $config->{REDIRECT_TO_NETWORK_PRINTERS}
  ) {
    my $tmp = delete $config->{REDIRECT_TO_NETWORK_PRINTERS};
    push @{$config->{REDIRECT_TO_NETWORK_PRINTERS}}, $tmp;
  }

  # convert the array of options into an hash by station num
  if (exists $config->{REDIRECT_TO_NETWORK_PRINTERS}) {
    my $ref = {};

    foreach my $redirect (@{$config->{REDIRECT_TO_NETWORK_PRINTERS}}) {
      my @a = split /,/, $redirect;

      if (scalar @a == 2) {
        $ref->{$a[0]} = $a[1];
      }
    }

    $config->{REDIRECT_TO_NETWORK_PRINTERS_BY_STATION} = $ref;
  }

  if (
    exists $config->{EXTERNAL_DOC} &&
    ! ref $config->{EXTERNAL_DOC}
  ) {
    my $tmp = delete $config->{EXTERNAL_DOC};
    push @{$config->{EXTERNAL_DOC}}, $tmp;
  }

  if ($config->{EXTERNAL_DOC} && scalar @{$config->{EXTERNAL_DOC}}) {
    foreach my $doc (@{$config->{EXTERNAL_DOC}}) {
      my @a = split /,/, $doc;
      $config->{EXTERNAL_DOC_BY_CODE}->{$a[0]} = $a[1];
    }
  }

  if (
    exists $config->{AREA_HAPPY_HOUR} &&
    ! ref $config->{AREA_HAPPY_HOUR}
  ) {
    my $tmp = delete $config->{AREA_HAPPY_HOUR};
    push @{$config->{AREA_HAPPY_HOUR}}, $tmp;
  }

  if (
    exists $config->{PrintingZones} &&
    exists $config->{PrintingZones}->{Zone} &&
    ref $config->{PrintingZones}->{Zone} eq 'HASH'
  ) {
    my $tmp = delete $config->{PrintingZones}->{Zone};
    push @{$config->{PrintingZones}->{Zone}}, $tmp;
  }

  $config->{PrintingZones}->{Zone} ||= [];

  foreach my $zone (@{ $config->{PrintingZones}->{Zone} }) {
    if (exists $zone->{Printer}) {
      if (ref($zone->{Printer}) eq 'HASH') {
        my $tmp = delete $zone->{Printer};
        push @{$zone->{Printer}}, $tmp;
      }
    }
  }

  if (exists $config->{Hardware}) {
    # scales
    if (
      exists $config->{Hardware}->{Scales} &&
      $config->{Hardware}->{Scales}->{Device} &&
      ref $config->{Hardware}->{Scales}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{Scales}->{Device};
      push @{$config->{Hardware}->{Scales}->{Device}}, $tmp;
    }

    # customer displays
    if (
      exists $config->{Hardware}->{CustomerDisplays} &&
      $config->{Hardware}->{CustomerDisplays}->{Device} &&
      ref $config->{Hardware}->{CustomerDisplays}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{CustomerDisplays}->{Device};
      push @{$config->{Hardware}->{CustomerDisplays}->{Device}}, $tmp;
    }

    # cashiers
    if (
      exists $config->{Cashiers} &&
      $config->{Cashiers}->{Cashier} &&
      ref $config->{Cashiers}->{Cashier} eq 'HASH'
    ) {
      my $tmp = delete $config->{Cashiers}->{Cashier};
      push @{$config->{Cashiers}->{Cashier}}, $tmp;
    }

    # barcode readers
    if (
      exists $config->{Hardware}->{BarcodeReaders} &&
      $config->{Hardware}->{BarcodeReaders}->{Device} &&
      ref $config->{Hardware}->{BarcodeReaders}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{BarcodeReaders}->{Device};
      push @{$config->{Hardware}->{BarcodeReaders}->{Device}}, $tmp;
    }

    # dallas readers
    if (
      exists $config->{Hardware}->{DallasReaders} &&
      $config->{Hardware}->{DallasReaders}->{Device} &&
      ref $config->{Hardware}->{DallasReaders}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{DallasReaders}->{Device};
      push @{$config->{Hardware}->{DallasReaders}->{Device}}, $tmp;
    }

    # queues
    if (
      exists $config->{Hardware}->{Queues} &&
      $config->{Hardware}->{Queues}->{Device} &&
      ref $config->{Hardware}->{Queues}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{Queues}->{Device};
      push @{$config->{Hardware}->{Queues}->{Device}}, $tmp;
    }

    # card readers - devices
    if (
      exists $config->{Hardware}->{CardReaders} &&
      $config->{Hardware}->{CardReaders}->{Device} &&
      ref $config->{Hardware}->{CardReaders}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{CardReaders}->{Device};
      push @{$config->{Hardware}->{CardReaders}->{Device}}, $tmp;
    }

    # cash drawers
    if (
      exists $config->{Hardware}->{CashDrawers} &&
      $config->{Hardware}->{CashDrawers}->{Device} &&
      ref $config->{Hardware}->{CashDrawers}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{CashDrawers}->{Device};
      push @{$config->{Hardware}->{CashDrawers}->{Device}}, $tmp;
    }

    # printers
    if (
      exists $config->{Hardware}->{Printers} &&
      $config->{Hardware}->{Printers}->{Device} &&
      ref $config->{Hardware}->{Printers}->{Device} eq 'HASH'
    ) {
      my $tmp = delete $config->{Hardware}->{Printers}->{Device};
      push @{$config->{Hardware}->{Printers}->{Device}}, $tmp;
    }
  }

}

sub load {
  my $self = shift;
  return $self->__cache() if $self->__cache();

  $self->debug("load");

  my $conf = Config::General->new($self->file());
  my %config = $conf->getall();

  $self->_init_array_fields(\%config);

  my $pt_certified_software = -e "/opt/pos/common/skins/".$config{LANG}."/sign_server" ? 1 : 0;

  $config{PRINT_BARCODE_IN_BILL} ||= 0;
  $config{PRINT_BARCODE_IN_CHECK} ||= 0;
  #$config{PRINT_BARCODE_IN_REQUEST} ||= 0;
  $config{CLOSE_DRAWER_MANDATORY} ||= 0;

  if (! $pt_certified_software && $config{LANG} ne 'ECR') {
    $config{DOC_TYPE_SELL} ||= 'VD';
    $config{DOC_TYPE_RETURN} ||= 'NCD';
    $config{DOC_TYPE_RETURN_NOTE} ||= 'GD';
    $config{DOC_TYPE_UPFRONT} ||= 'AD';
    $config{DOC_TYPE_VOID} ||= 'VOID';
    $config{DOC_TYPE_CONSUMPTION} ||= 'CP';
    $config{DOC_TYPE_SELL_CUSTOMER} ||= 'FR';
    $config{DOC_TYPE_SELL_SIMPLIFIED} ||= 'FS';
    $config{DOC_TYPE_TABLE_PRINT} ||= 'CM';
    $config{DOC_TYPE_TABLE_PRINT_INTERNAL} ||= 'CM2';
    $config{DOC_TYPE_CUSTOMER_CREDIT} ||= 'FT';
    $config{DOC_TYPE_RECEIPT} ||= 'RG';
    $config{DOC_TYPE_TRANSPORT_NOTE} ||= 'GT';
    $config{DOC_TYPE_DELIVERY_NOTE} ||= 'GR';
    $config{DOC_TYPE_TRANSPORT_DOCUMENT} ||= 'DT';
    $config{DOC_TYPE_DEBIT_NOTE} ||= 'ND';
    $config{DOC_TYPE_CREDIT_NOTE} ||= 'NC';
    $config{DOC_TYPE_EXTERNAL} ||= 'EXT';
    $config{DOC_TYPE_SUPPLIER_FVD} ||= 'FVD';
    $config{DOC_TYPE_SUPPLIER_FFR} ||= 'FFR';
    $config{DOC_TYPE_SUPPLIER_FFT} ||= 'FFT';
    $config{DOC_TYPE_SUPPLIER_FNC} ||= 'FNC';
    $config{DOC_TYPE_SUPPLIER_FND} ||= 'FND';
    $config{DOC_TYPE_SUPPLIER_FNL} ||= 'FNL';
  }
  else {
    $config{DOC_TYPE_SELL} = 'VD';
    $config{DOC_TYPE_RETURN} = 'NCD';
    $config{DOC_TYPE_RETURN_NOTE} = 'GD';
    $config{DOC_TYPE_UPFRONT} = 'AD';
    $config{DOC_TYPE_VOID} = 'VOID';
    $config{DOC_TYPE_CONSUMPTION} = 'CP';
    $config{DOC_TYPE_SELL_CUSTOMER} = 'FR';
    $config{DOC_TYPE_SELL_SIMPLIFIED} = 'FS';
    $config{DOC_TYPE_TABLE_PRINT} = 'CM';
    $config{DOC_TYPE_TABLE_PRINT_INTERNAL} = 'CM2';
    $config{DOC_TYPE_CUSTOMER_CREDIT} = 'FT';
    $config{DOC_TYPE_RECEIPT} = 'RG';
    $config{DOC_TYPE_TRANSPORT_NOTE} = 'GT';
    $config{DOC_TYPE_DELIVERY_NOTE} = 'GR';
    $config{DOC_TYPE_TRANSPORT_DOCUMENT} = 'DT';
    $config{DOC_TYPE_DEBIT_NOTE} = 'ND';
    $config{DOC_TYPE_CREDIT_NOTE} = 'NC';
    $config{DOC_TYPE_EXTERNAL} = 'EXT';
    $config{DOC_TYPE_SUPPLIER_FVD} = 'FVD';
    $config{DOC_TYPE_SUPPLIER_FFR} = 'FFR';
    $config{DOC_TYPE_SUPPLIER_FFT} = 'FFT';
    $config{DOC_TYPE_SUPPLIER_FNC} = 'FNC';
    $config{DOC_TYPE_SUPPLIER_FND} = 'FND';
    $config{DOC_TYPE_SUPPLIER_FNL} = 'FNL';
  }

  # remove spaces from here
  $config{DOC_TYPE_SERIE} =~ s/ //g;

  $config{PRINT_DOC_TYPE_SELL} ||= 1;
  $config{PRINT_DOC_TYPE_RETURN} ||= 1;
  $config{PRINT_DOC_TYPE_RETURN_NOTE} ||= 1;
  $config{PRINT_DOC_TYPE_UPFRONT} ||= 1;
  $config{PRINT_DOC_TYPE_VOID} ||= 1;
  $config{PRINT_DOC_TYPE_CONSUMPTION} ||= 1;
  $config{PRINT_DOC_TYPE_SELL_CUSTOMER} ||= 1;
  $config{PRINT_DOC_TYPE_SELL_SIMPLIFIED} ||= 1;
  $config{PRINT_DOC_TYPE_TABLE_PRINT} ||= 1;
  $config{PRINT_DOC_TYPE_TABLE_PRINT_INTERNAL} ||= 1;
  $config{PRINT_DOC_TYPE_CUSTOMER_CREDIT} ||= 1;
  $config{PRINT_DOC_TYPE_CUSTOMER_CREDIT_CC} ||= 1;
  $config{PRINT_DOC_TYPE_RECEIPT} ||= 1;
  $config{PRINT_DOC_TYPE_TRANSPORT_NOTE} ||= 1;
  $config{PRINT_DOC_TYPE_DELIVERY_NOTE} ||= 1;
  $config{PRINT_DOC_TYPE_DEBIT_NOTE} ||= 1;
  $config{PRINT_DOC_TYPE_CREDIT_NOTE} ||= 1;
  $config{PRINT_DOC_TYPE_EXTERNAL} ||= 1;
  $config{PRINT_CC_REPORTS} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FVD} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FFR} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FFT} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FNC} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FND} ||= 1;
  $config{PRINT_DOC_TYPE_SUPPLIER_FNL} ||= 1;

  $config{SALES_TO_NETWORK} ||= 0;
  $config{MACHINE_LABEL} ||= "";

  if (! exists $config{DONT_CHANGE_DAY_AFTER_DAY_CLOSE}) {
    $config{DONT_CHANGE_DAY_AFTER_DAY_CLOSE} = 1;
  }

  $self->__cache(\%config);

  return $self->__cache();
}

sub areas_info {
  my $self = shift;
  return $self->{__areas_info} if exists $self->{__areas_info};

  my %areas_info = ();

  my $config = $self->load_fixed();

  $config->{AREA} ||= [];

  my $area_idx = 0;

  foreach my $area (@{ $config->{AREA} }) {
    my @t = split /,/, $area;
    $areas_info{$area_idx++} = \@t;
  }

  $self->{__areas_info} = \%areas_info;
  return $self->{__areas_info};
}

sub printing_zones_by_name {
  my $self = shift;
  return $self->{__zones_by_name} if exists $self->{__zones_by_name};

  my $cfg = $self->load_fixed();
  $cfg->{PrintingZones}->{Zone} ||= [];
  return unless scalar @{ $cfg->{PrintingZones}->{Zone} };

  my $z = $cfg->{PrintingZones}->{Zone};
  my $h = {};

  foreach my $zone (@$z) {
    $h->{$zone->{Name}} = $zone;
  }

  $self->{__zones_by_name} = $h;
  return $h;
}

sub printers_info {
  my $self = shift;
  return $self->{__printers_info} if exists $self->{__printers_info};

  my $i = {};

  my $cfg = $self->load_fixed();

  $cfg->{PrintingZones}->{Zone} ||= [];

  foreach my $zone (@{ $cfg->{PrintingZones}->{Zone} }) {
    $zone->{Printer} ||= [];

    foreach my $printer (@{$zone->{Printer}}) {
      $printer->{FromArea} //= "";
      $printer->{FromStation} //= "";

      if (! exists $i->{ $zone->{Name} }->{ $printer->{Station} }->{ $printer->{Number} }) {
        $i->{ $zone->{Name} }->{ $printer->{Station} }->{ $printer->{Number} } = $printer;
      }

      if (exists $printer->{Template} && defined $printer->{Template} && $printer->{Template} ne '') {
        if (! exists $printer->{FromStation}) {
          $printer->{FromStation} = "";
        }

        if (! exists $printer->{FromArea}) {
          $printer->{FromArea} = "";
        }

        if ($printer->{FromArea} ne '' || $printer->{FromStation} ne '') {
          my $key = $printer->{FromArea}."#".$printer->{FromStation};
          $i->{ $zone->{Name} }->{ $printer->{Station} }->{ $printer->{Number} }->{TemplateFilter}->{$key} = $printer->{Template};
        }
      }
    }
  }

  $self->{__printers_info} = $i;

  return $i;
}

sub doc_type_str_by_id {
  my $self = shift;
  my $id = shift;

  my $h = $self->doc_types_by_id();

  return exists $h->{$id} ? $h->{$id} : "";
}

sub doc_types_by_name {
  my $self = shift;

  my $cfg = $self->load();

  my %i = (
    $cfg->{DOC_TYPE_SELL}                 => 1,
    $cfg->{DOC_TYPE_RETURN}               => 2,
    $cfg->{DOC_TYPE_VOID}                 => 3,
    $cfg->{DOC_TYPE_CONSUMPTION}          => 4,
    $cfg->{DOC_TYPE_SELL_CUSTOMER}        => 5,
    $cfg->{DOC_TYPE_TABLE_PRINT}          => 6,
    $cfg->{DOC_TYPE_CUSTOMER_CREDIT}      => 7,
    $cfg->{DOC_TYPE_RECEIPT}              => 8,
    $cfg->{DOC_TYPE_TABLE_PRINT_INTERNAL} => 9,
    $cfg->{DOC_TYPE_TRANSPORT_NOTE}       => 10,
    $cfg->{DOC_TYPE_DELIVERY_NOTE}        => 11,
    $cfg->{DOC_TYPE_DEBIT_NOTE}           => 12,
    $cfg->{DOC_TYPE_CREDIT_NOTE}          => 13,
    $cfg->{DOC_TYPE_UPFRONT}              => 14,
    $cfg->{DOC_TYPE_RETURN_NOTE}          => 15,
    $cfg->{DOC_TYPE_EXTERNAL}             => 99,
    $cfg->{DOC_TYPE_SELL_SIMPLIFIED}      => 90,

    $cfg->{DOC_TYPE_SUPPLIER_FVD}         => 30,
    $cfg->{DOC_TYPE_SUPPLIER_FFR}         => 31,
    $cfg->{DOC_TYPE_SUPPLIER_FFT}         => 32,
    $cfg->{DOC_TYPE_SUPPLIER_FNC}         => 33,
    $cfg->{DOC_TYPE_SUPPLIER_FND}         => 34,
    $cfg->{DOC_TYPE_SUPPLIER_FNL}         => 35,
  );

  return wantarray ? %i : \%i;
}

sub doc_types_by_id {
  my $self = shift;

  my $cfg = $self->load();

  my %i = (
    1  => $cfg->{DOC_TYPE_SELL},
    2  => $cfg->{DOC_TYPE_RETURN},
    3  => $cfg->{DOC_TYPE_VOID},
    4  => $cfg->{DOC_TYPE_CONSUMPTION},
    5  => $cfg->{DOC_TYPE_SELL_CUSTOMER},
    6  => $cfg->{DOC_TYPE_TABLE_PRINT},
    7  => $cfg->{DOC_TYPE_CUSTOMER_CREDIT},
    8  => $cfg->{DOC_TYPE_RECEIPT},
    9  => $cfg->{DOC_TYPE_TABLE_PRINT_INTERNAL},
    10 => $cfg->{DOC_TYPE_TRANSPORT_NOTE},
    11 => $cfg->{DOC_TYPE_DELIVERY_NOTE},
    12 => $cfg->{DOC_TYPE_DEBIT_NOTE},
    13 => $cfg->{DOC_TYPE_CREDIT_NOTE},
    14 => $cfg->{DOC_TYPE_UPFRONT},
    15 => $cfg->{DOC_TYPE_RETURN_NOTE},
    99 => $cfg->{DOC_TYPE_EXTERNAL},
    90 => $cfg->{DOC_TYPE_SELL_SIMPLIFIED},

    30 => $cfg->{DOC_TYPE_SUPPLIER_FVD},
    31 => $cfg->{DOC_TYPE_SUPPLIER_FFR},
    32 => $cfg->{DOC_TYPE_SUPPLIER_FFT},
    33 => $cfg->{DOC_TYPE_SUPPLIER_FNC},
    34 => $cfg->{DOC_TYPE_SUPPLIER_FND},
    35 => $cfg->{DOC_TYPE_SUPPLIER_FNL},
  );

  return wantarray ? %i : \%i;
}

sub print_destination_by_docid {
  my $self = shift;

  my $cfg = $self->load();

  my %i = (
    1  => $cfg->{PRINT_DOC_TYPE_SELL},
    2  => $cfg->{PRINT_DOC_TYPE_RETURN},
    3  => $cfg->{PRINT_DOC_TYPE_VOID},
    4  => $cfg->{PRINT_DOC_TYPE_CONSUMPTION},
    5  => $cfg->{PRINT_DOC_TYPE_SELL_CUSTOMER},
    6  => $cfg->{PRINT_DOC_TYPE_TABLE_PRINT},
    7  => $cfg->{PRINT_DOC_TYPE_CUSTOMER_CREDIT},
    71 => $cfg->{PRINT_DOC_TYPE_CUSTOMER_CREDIT_CC},
    8  => $cfg->{PRINT_DOC_TYPE_RECEIPT},
    9  => $cfg->{PRINT_DOC_TYPE_TABLE_PRINT_INTERNAL},
    10 => $cfg->{PRINT_DOC_TYPE_TRANSPORT_NOTE},
    11 => $cfg->{PRINT_DOC_TYPE_DELIVERY_NOTE},
    12 => $cfg->{PRINT_DOC_TYPE_DEBIT_NOTE},
    13 => $cfg->{PRINT_DOC_TYPE_CREDIT_NOTE},
    14 => $cfg->{PRINT_DOC_TYPE_UPFRONT},
    15 => $cfg->{PRINT_DOC_TYPE_RETURN_NOTE},
    99 => $cfg->{PRINT_DOC_TYPE_EXTERNAL},
    90 => $cfg->{PRINT_DOC_TYPE_SELL_SIMPLIFIED},
    30 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FVD},
    31 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FFR},
    32 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FFT},
    33 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FNC},
    34 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FND},
    35 => $cfg->{PRINT_DOC_TYPE_SUPPLIER_FNL},
  );

  return wantarray ? %i : \%i;
}

sub print_template_by_docid {
  my $self = shift;

  my $cfg = $self->load();

  my %i = (
    1  => 'print_invoice',
    2  => 'print_return',
    3  => '',
    4  => 'print_consumption',
    5  => 'print_invoice',
    6  => 'print_bill',
    7  => 'print_invoice',
    8  => 'print_receipt',
    9  => 'print_bill_internal',
    10 => 'print_transport_note',
    11 => 'print_delivery_note',
    12 => 'print_debit_note',
    13 => 'print_credit_note',
    14 => 'print_upfront',
    15 => 'print_return_note',
    99 => 'print_external',

    30 => 'print_invoice',
    31 => 'print_invoice',
    32 => 'print_invoice',
    33 => 'print_invoice',
    34 => 'print_invoice',
    35 => 'print_receipt',
  );

  return wantarray ? %i : \%i;
}

sub print_destination {
  my $self = shift;
  my $doc_type = shift;

  my $print_destination_by_docid = $self->print_destination_by_docid();

  return exists $print_destination_by_docid->{$doc_type} ? $print_destination_by_docid->{$doc_type} : 1;
}

sub document_template {
  my $self = shift;
  my( $dir, $doc_type, $overrided_destination ) = @_;

  my $templates = $self->print_template_by_docid();
  my $print_destination_by_docid = $self->print_destination_by_docid();

  my $print_destination = exists $print_destination_by_docid->{$doc_type} ? $print_destination_by_docid->{$doc_type} : 1;
  my $template_file = exists $templates->{$doc_type} ? $templates->{$doc_type} : 'print_invoice';

  # if an invalid template is being used, use the print_invoice as default
  if (! -e $dir.'/common/print_templates/'.$template_file.".tmpl") {
    $template_file = 'print_invoice';
  }

  # this is an optional parameter that can be used to override the print destination.
  # ex, the doc would be printed to an external printer and we want it to go to the ticket one
  if (defined $overrided_destination) {
    $print_destination = $overrided_destination;
  }

  # 1 = to receipt printer
  # 2 = to external printer
  if ($print_destination == 2) {
    $template_file .= ".external";
  }

  my $cfg = $self->load();
  my $skin = $cfg->{LANG};

  my $tmpl_file = "";

  if (-e $dir.'/common/print_templates/'.$cfg->{LANG}."/$template_file.tmpl") {
    $tmpl_file = $cfg->{LANG}."/$template_file.tmpl"
  } else {
    $tmpl_file = $template_file.'.tmpl';
  }

  return $tmpl_file;
}


sub load_fixed {
  my $self = shift;

  if (exists $self->{__config_fixed_loaded}) {
    return $self->{__config_fixed_loaded};
  }

  my $h = $self->load();

  $self->_init_array_fields($h);

  $self->{__config_fixed_loaded} = $h;

  return $h;
}

sub devices_by_type {
  my $self = shift;
  my $type = shift;

  # invalidate config cache
  undef $self->{__cache};

  my $info = $self->load_fixed();
  my @devices = ();

  if (exists $info->{Hardware} && exists $info->{Hardware}->{$type}) {
    my $idx = 0;
    foreach my $device ( @{ $info->{Hardware}->{$type}->{Device} } ) {
      my $num = $idx;

      push @devices, {
        num => $num,
        device => $device,
      };

      $idx++;
    }
  }

  return \@devices;
}

sub __fix {
}

sub __cache {
  my $self = shift;
  @_ ? $self->{__cache} = shift : $self->{__cache};
}


1;
