package App::POS::Hardware::Scales::CAS_ER_PLUS;

use warnings;
use strict;
use Time::HiRes qw( sleep );
use App::POS::Hardware::Scales;

use base qw( App::POS::Hardware::Scales );

sub setup_port {
  my $s = shift;

  my $PortObj = $s->serial_obj();
  $PortObj->baudrate(9600);
  $PortObj->parity("even");
  $PortObj->databits(8);
  $PortObj->stopbits(1);
  $PortObj->handshake("none");
}

sub start {
  my $s = shift;

  my $PortObj = $s->serial_obj();

  my $info = $s->info();

  $info->{Aux1} ||= '';
  $info->{Aux2} ||= '';

  my $sleep = 0.1;
  my $wrong = 0;
  my $last_read = -1;
  my %read_control = ();

  if ($info->{Aux1}) {
    $sleep = $info->{Aux1};
  }

  while (1) {
    $PortObj->write(chr(0x05));
    my( $count_in, $string_in ) = $PortObj->read(1);
    $PortObj->write(chr(0x11).chr(0x12));
    ( $count_in, $string_in ) = $PortObj->read(1024);

    $s->debug("chars read: $count_in, string_read: #$string_in#");

    my $num = 0;

    if ($count_in > 0) {
      ( $num ) = $string_in =~ /(\d+\.\d+)kg/;
      $num ||= 0;
    }

    $num = sprintf("%.3f", $num);

    #
    # tweaked reading using the Aux1 and Aux2 field for loop delay and stabalize count
    #
    if ($info->{Aux2}) {
      if ($count_in > 3 && $string_in) {
        if ($num != $last_read) {
          if (! exists $read_control{$num}) {
            %read_control = ();
          }
          $read_control{$num}++;
          if ($read_control{$num} * 1 == $info->{Aux2} * 1) {
            open(N, "> /opt/pos/common/tmp/scale.txt");
            print N sprintf("%.3f", $num)."\n";
            close(N);
            $last_read = $num;
            %read_control = ();
          }
        }
      }
    }
    #
    # default reading
    #
    else {
      if ($count_in > 0) {
        if ($wrong == 1) {
          open(N, "> /opt/pos/common/tmp/scale.txt");
          print N "0.000\n";
          close(N);
        }

        $num ||= 0;

        if ($num > 0) {
          $wrong = 0;
          open(N, "> /opt/pos/common/tmp/scale.txt");
          print N sprintf("%.3f", $num)."\n";
          close(N);
        }
        else {
          $wrong++;
        }
      }
    }

    sleep($sleep);
  }
}

1;

