package App::POS::Hardware::Scales::CAS_PR_II;
use warnings;
use strict;
use Data::Dumper;
use Time::HiRes qw( sleep );
use App::POS::Hardware::Scales;

use base qw( App::POS::Hardware::Scales );

sub setup_port {
  my $s = shift;

  my $PortObj = $s->serial_obj();
  $PortObj->baudrate(9600);
  $PortObj->parity("none");
  $PortObj->databits(8);
  $PortObj->stopbits(1);
  $PortObj->handshake("none");
}

sub start {
  my $s = shift;

  my $PortObj = $s->serial_obj();

  my $info = $s->info();

  $info->{Aux1} ||= '';
  $info->{Aux2} ||= '';

  my $sleep = 0.1;
  my $wrong = 0;
  my $last_read = -1;
  my %read_control = ();

  if ($info->{Aux1}) {
    $sleep = $info->{Aux1};
  }

  #SOH  0x01
  #STX  0x02
  #ETX  0x03
  #EOT  0x04
  #CR   0x0d
  #LF   0x0a
  #RS   0x1e
  #ENQ  0x05
  #NAK  0x15
  #NULL 0x00
  #DC1  0x11
  #DC2  0x12
  #CAN  0x18

  while (1) {
    # print STDERR "loop\n";
    $PortObj->write(chr(0x05));
    my( $count_in, $string_in ) = $PortObj->read(1);
    $count_in ||= 0;

    $s->debug("chars read: $count_in, string_read: #$string_in#");

    if ($count_in > 0) {
      # print STDERR "##11 $string_in ##\n";

      if (ord($string_in) == 0x15) {
        # print STDERR "error reading...\n";
      }
      else {
        # read mode 1 (only gets weight)
        $PortObj->write(chr(0x11));

        # read mode 2 (includes eur and eur/kg)
        #$PortObj->write(chr(0x12));

        my( $count_in, $string_in ) = $PortObj->read(255);

        my $num = 0;

        if ($count_in > 0) {
          ( $num ) = $string_in =~ /(\d+\.\d+)kg/;
          $num ||= 0;
        }

        $num = sprintf("%.3f", $num);

        #
        # tweaked reading using the Aux1 and Aux2 field for loop delay and stabalize count
        #
        if ($info->{Aux2}) {
          if ($count_in > 3 && $string_in) {
            if ($num != $last_read) {
              if (! exists $read_control{$num}) {
                %read_control = ();
              }
              $read_control{$num}++;
              if ($read_control{$num} * 1 == $info->{Aux2} * 1) {
                open(N, "> /opt/pos/common/tmp/scale.txt");
                print N sprintf("%.3f", $num)."\n";
                close(N);
                $last_read = $num;
                %read_control = ();
              }
            }
          }
        }
        #
        # default reading
        #
        else {
          if ($count_in > 0) {
            if ($wrong == 1) {
              open(N, "> /opt/pos/common/tmp/scale.txt");
              print N "0.000\n";
              close(N);
            }

            $num ||= 0;

            if ($num > 0) {
              $wrong = 0;
              open(N, "> /opt/pos/common/tmp/scale.txt");
              print N sprintf("%.3f", $num)."\n";
              close(N);
            }
            else {
              $wrong++;
            }
          }
        }
      }
    }

    sleep($sleep);
  }
}

1;

